@@ -5,34 +5,12 @@ NULL
 
 check_period <- function(object) {
   errors <- character()
-  ## if (!is.numeric(object@.Data)) {
-  ##   msg <- "seconds (.Data) value must be numeric."
-  ##   errors <- c(errors, msg)
-  ## }
-  ## if (!is.numeric(object@year)) {
-  ##   msg <- "year value must be numeric."
-  ##   errors <- c(errors, msg)
-  ## }
-  ## if (!is.numeric(object@month)) {
-  ##   msg <- "year value must be numeric."
-  ##   errors <- c(errors, msg)
-  ## }
-  ## if (!is.numeric(object@day)) {
-  ##   msg <- "year value must be numeric."
-  ##   errors <- c(errors, msg)
-  ## }
-  ## if (!is.numeric(object@hour)) {
-  ##   msg <- "year value must be numeric."
-  ##   errors <- c(errors, msg)
-  ## }
-  ## if (!is.numeric(object@minute)) {
-  ##   msg <- "year value must be numeric."
-  ##   errors <- c(errors, msg)
-  ## }
 
   length(object@.Data) -> n
-  lengths <- c(length(object@year), length(object@month),
-               length(object@day), length(object@hour), length(object@minute))
+  lengths <- c(
+    length(object@year), length(object@month),
+    length(object@day), length(object@hour), length(object@minute)
+  )
 
   if (any(lengths != n)) {
     msg <- paste("Inconsistent lengths: year = ", lengths[1],
@@ -41,7 +19,8 @@ check_period <- function(object) {
       ", hour = ", lengths[4],
       ", minute = ", lengths[5],
       ", second = ", n,
-      sep = "")
+      sep = ""
+    )
     errors <- c(errors, msg)
   }
 
@@ -52,10 +31,11 @@ check_period <- function(object) {
     errors <- c(errors, msg)
   }
 
-  if (length(errors) == 0)
+  if (length(errors) == 0) {
     TRUE
-  else
+  } else {
     errors
+  }
 }
 
 #' Period class
@@ -81,7 +61,7 @@ check_period <- function(object) {
 #'
 #' Because Period represents imprecise amount of time it cannot be compared to
 #' precise timestamps as Durations and Intervals are. You need to explicitely
-#' conver to durations. See [Duration-class].
+#' convert to durations. See [Duration-class].
 #'
 #' The logic that guides arithmetic with periods can be unintuitive. Starting
 #' with version 1.3.0, \pkg{lubridate} enforces the reversible property of arithmetic
@@ -107,11 +87,15 @@ check_period <- function(object) {
 #'
 #' @export
 #' @keywords internal
-setClass("Period", contains = c("Timespan", "numeric"),
-  slots = c(year = "numeric", month = "numeric", day = "numeric",
-    hour = "numeric", minute = "numeric"),
+setClass("Period",
+  contains = c("Timespan", "numeric"),
+  slots = c(
+    year = "numeric", month = "numeric", day = "numeric",
+    hour = "numeric", minute = "numeric"
+  ),
   prototype = prototype(year = 0, month = 0, day = 0, hour = 0, minute = 0),
-  validity = check_period)
+  validity = check_period
+)
 
 #' @name hidden_aliases
 #' @aliases Arith,ANY,Period-method Arith,Duration,Period-method
@@ -120,81 +104,113 @@ setClass("Period", contains = c("Timespan", "numeric"),
 #'   Compare,Period,difftime-method Compare,Period,Period-method
 #'   Compare,Period,character-method Compare,Period,numeric-method
 #'   Compare,character,Period-method second,Period-method second<-,Period-method
-#'   minute,Period-method minute<-,Period-method hour,Period-method
-#'   hour<-,Period-method Arith,Period,ANY-method day,Period-method
-#'   day<-,Period-method month,Period-method month<-,Period-method
-#'   year,Period-method year<-,Period-method date,Period-method
-#'   date<-,Period-method as.numeric,Period-method show,Period-method
-#'   c,Period-method rep,Period-method [,Period-method
-#'   [<-,Period,ANY,ANY,Period-method [[,Period-method
-#'   [[<-,Period,ANY,ANY,Period-method $,Period-method $<-,Period-method
-#'   as.difftime,Period-method as.character,Period-method
-#'   +,Period,Duration-method +,Period,Interval-method +,Period,Period-method
-#'   +,Period,Date-method +,Date,Period-method +,Period,difftime-method
-#'   +,difftime,Period-method +,Period,numeric-method +,numeric,Period-method
-#'   +,Period,POSIXct-method +,POSIXct,Period-method +,Period,POSIXlt-method
-#'   +,POSIXlt,Period-method /,Period,Duration-method /,Period,Interval-method
-#'   /,Period,Period-method /,Period,difftime-method /,difftime,Period-method
-#'   /,Period,numeric-method /,numeric,Period-method *,Period,ANY-method
-#'   *,ANY,Period-method -,Period,ANY-method -,Period,missing-method
-#'   -,ANY,Period-method %%,Period,Duration-method %%,Period,Interval-method
-#'   %%,Period,Period-method >,Period,Period-method >=,Period,Period-method
-#'   ==,Period,Period-method !=,Period,Period-method <=,Period,Period-method
-#'   <,Period,Period-method >,Period,Duration-method >=,Period,Duration-method
-#'   ==,Period,Duration-method !=,Period,Duration-method
+#'   minute,Period-method minute<-,Period-method hour,Period-method hour<-,Period-method
+#'   Arith,Period,ANY-method day,Period-method day<-,Period-method month,Period-method
+#'   month<-,Period-method year,Period-method year<-,Period-method date,Period-method
+#'   date<-,Period-method as.numeric,Period-method as.integer,Period-method
+#'   show,Period-method c,Period-method rep,Period-method [,Period-method
+#'   [<-,Period,ANY,ANY,Period-method [[,Period-method [[<-,Period,ANY,ANY,Period-method
+#'   $,Period-method $<-,Period-method as.difftime,Period-method
+#'   as.character,Period-method +,Period,Duration-method +,Period,Interval-method
+#'   +,Period,Period-method +,Period,Date-method +,Date,Period-method
+#'   +,Period,difftime-method +,difftime,Period-method +,Period,numeric-method
+#'   +,numeric,Period-method +,Period,POSIXct-method +,POSIXct,Period-method
+#'   +,Period,POSIXlt-method +,POSIXlt,Period-method /,Period,Duration-method
+#'   /,Period,Interval-method /,Period,Period-method /,Period,difftime-method
+#'   /,difftime,Period-method /,Period,numeric-method /,numeric,Period-method
+#'   *,Period,ANY-method *,ANY,Period-method -,Period,ANY-method -,Period,missing-method
+#'   -,ANY,Period-method -,Period,Period-method %%,Period,Duration-method
+#'   %%,Period,Interval-method %%,Period,Period-method >,Period,Period-method
+#'   >=,Period,Period-method ==,Period,Period-method !=,Period,Period-method
+#'   <=,Period,Period-method <,Period,Period-method >,Period,Duration-method
+#'   >=,Period,Duration-method ==,Period,Duration-method !=,Period,Duration-method
 #'   <=,Period,Duration-method <,Period,Duration-method >,Duration,Period-method
-#'   >=,Duration,Period-method ==,Duration,Period-method
-#'   !=,Duration,Period-method <=,Duration,Period-method
-#'   <,Duration,Period-method >,Period,numeric-method >=,Period,numeric-method
-#'   ==,Period,numeric-method !=,Period,numeric-method <=,Period,numeric-method
-#'   <,Period,numeric-method >,numeric,Period-method >=,numeric,Period-method
-#'   ==,numeric,Period-method !=,numeric,Period-method <=,numeric,Period-method
-#'   <,numeric,Period-method !=,Duration,Period !=,Period,Duration
-#'   !=,Period,Period !=,Period,numeric !=,numeric,Period %%,Period,Duration
-#'   %%,Period,Interval %%,Period,Period *,ANY,Period *,Period,ANY -,ANY,Period
-#'   -,Period,Interval -,Period,missing /,numeric,Period <,Duration,Period
+#'   >=,Duration,Period-method ==,Duration,Period-method !=,Duration,Period-method
+#'   <=,Duration,Period-method <,Duration,Period-method >,Period,numeric-method
+#'   >=,Period,numeric-method ==,Period,numeric-method !=,Period,numeric-method
+#'   <=,Period,numeric-method <,Period,numeric-method >,numeric,Period-method
+#'   >=,numeric,Period-method ==,numeric,Period-method !=,numeric,Period-method
+#'   <=,numeric,Period-method <,numeric,Period-method !=,Duration,Period
+#'   !=,Period,Duration !=,Period,Period !=,Period,numeric !=,numeric,Period
+#'   %%,Period,Duration %%,Period,Interval %%,Period,Period *,ANY,Period *,Period,ANY
+#'   -,ANY,Period -,Period,Interval -,Period,missing /,numeric,Period <,Duration,Period
 #'   <,Period,Duration <,Period,Period <,Period,numeric <,numeric,Period
 #'   <=,Duration,Period <=,Period,Duration <=,Period,Period <=,Period,numeric
 #'   <=,numeric,Period ==,Duration,Period ==,Period,Duration ==,Period,Period
 #'   ==,Period,numeric ==,numeric,Period >,Duration,Period >,Period,Duration
 #'   >,Period,Period >,Period,numeric >,numeric,Period >=,Duration,Period
 #'   >=,Period,Duration >=,Period,Period >=,Period,numeric >=,numeric,Period
+#'   -,Period,Date-method -,Period,Duration-method -,Period,POSIXct-method
+#'   -,Period,POSIXlt-method -,Period,numeric-method -,Date,Period-method
+#'   -,numeric,Period-method -,POSIXlt,Period-method -,POSIXct,Period-method
 NULL
 
 setMethod("initialize", "Period", function(.Object, ...) {
   dots <- list(...)
   names(dots)[!nzchar(allNames(dots))] <- ".Data"
-  len <- max(unlist(lapply(dots, length), F, F))
+  lens <- unlist(lapply(dots, length), F, F)
+
+  ## if any 0-length components, the entire object is 0-length
+  if (any(lens == 0)) {
+    for (nm in slotNames(.Object)) {
+      slot(.Object, nm) <- numeric()
+    }
+    validObject(.Object)
+    return(.Object)
+  }
+
+  len <- max(lens)
+  nas <- FALSE
   for (nm in slotNames(.Object)) {
-    slot(.Object, nm) <-
+    el <-
       if (is.null(obj <- dots[[nm]])) {
         rep.int(0L, len)
       } else {
-        if (length(obj) < len) rep_len(obj, len)
-        else obj
+        if (length(obj) < len) {
+          rep_len(obj, len)
+        } else {
+          obj
+        }
       }
+    nas <- nas | is.na(el)
+    slot(.Object, nm) <- el
   }
+
+  ## If any component has NAs the entire object should have those NAs
+  if (any(nas)) {
+    for (nm in slotNames(.Object)) {
+      slot(.Object, nm)[nas] <- NA_real_
+    }
+  }
+
   validObject(.Object)
   .Object
 })
 
 #' @export
 setMethod("show", signature(object = "Period"), function(object) {
-  print(format(object))
+  if (length(object@.Data) == 0) {
+    cat("<Period[0]>\n")
+  } else {
+    print(format(object))
+  }
 })
 
 #' @export
 format.Period <- function(x, ...) {
-  if (length(x@.Data) == 0) return("Period(0)")
-  show <- vector(mode = "character")
-  na <- is.na(x)
+  if (length(x) == 0) {
+    return(character())
+  }
 
-  show <- paste(x@year, "y ", x@month, "m ", x@day, "d ",
-    x@hour, "H ", x@minute, "M ", x@.Data, "S", sep = "")
+  show <- paste(
+    x@year, "y ", x@month, "m ", x@day, "d ",
+    x@hour, "H ", x@minute, "M ", x@.Data, "S",
+    sep = ""
+  )
   start <- regexpr("[-1-9]|(0\\.)", show)
   show <- ifelse(start > 0, substr(show, start, nchar(show)), "0S")
 
-  show[na] <- NA
+  show[is.na(x)] <- NA
   show
 }
 
@@ -204,41 +220,58 @@ xtfrm.Period <- function(x) {
 }
 
 #' @export
-setMethod("c", signature(x = "Period"), function(x, ...) {
-  elements <- lapply(list(...), as.period)
-  seconds <- c(x@.Data, unlist(lapply(elements, slot, ".Data")))
-  years <- c(x@year, unlist(lapply(elements, slot, "year")))
-  months <- c(x@month, unlist(lapply(elements, slot, "month")))
-  days <- c(x@day, unlist(lapply(elements, slot, "day")))
-  hours <- c(x@hour, unlist(lapply(elements, slot, "hour")))
-  minutes <- c(x@minute, unlist(lapply(elements, slot, "minute")))
-  new("Period", seconds, year = years, month = months, day = days,
-    hour = hours, minute = minutes)
-})
+c.Period <- function(...) {
+  dots <- list(...)
+  nempty <- sapply(dots, length) != 0
+  elements <- lapply(dots[nempty], as.period)
+  seconds <- unlist(lapply(elements, slot, ".Data"))
+  # don't keep names for slots
+  elements <- unname(elements)
+  years <- unlist(lapply(elements, slot, "year"))
+  months <- unlist(lapply(elements, slot, "month"))
+  days <- unlist(lapply(elements, slot, "day"))
+  hours <- unlist(lapply(elements, slot,  "hour"))
+  minutes <- unlist(lapply(elements, slot, "minute"))
+  new("Period",
+    seconds, year = years, month = months,
+    day = days, hour = hours, minute = minutes
+  )
+}
 
 #' @export
 setMethod("rep", signature(x = "Period"), function(x, ...) {
-  new("Period", rep(x@.Data, ...), year = rep(x@year, ...),
+  new("Period", rep(x@.Data, ...),
+    year = rep(x@year, ...),
     month = rep(x@month, ...), day = rep(x@day, ...),
-    hour = rep(x@hour, ...), minute = rep(x@minute, ...))
+    hour = rep(x@hour, ...), minute = rep(x@minute, ...)
+  )
 })
 
 #' @export
-setMethod("[", signature(x = "Period"),
+setMethod(
+  "[", signature(x = "Period"),
   function(x, i, j, ..., drop = TRUE) {
-    new("Period", x@.Data[i], year = x@year[i], month = x@month[i],
-      day = x@day[i], hour = x@hour[i], minute = x@minute[i])
-})
+    new("Period", x@.Data[i],
+      year = x@year[i], month = x@month[i],
+      day = x@day[i], hour = x@hour[i], minute = x@minute[i]
+    )
+  }
+)
 
 #' @export
-setMethod("[[", signature(x = "Period"),
-          function(x, i, j, ..., exact = TRUE) {
-            new("Period", x@.Data[i], year = x@year[i], month = x@month[i],
-                day = x@day[i], hour = x@hour[i], minute = x@minute[i])
-})
+setMethod(
+  "[[", signature(x = "Period"),
+  function(x, i, j, ..., exact = TRUE) {
+    new("Period", x@.Data[i],
+      year = x@year[i], month = x@month[i],
+      day = x@day[i], hour = x@hour[i], minute = x@minute[i]
+    )
+  }
+)
 
 #' @export
-setMethod("[<-", signature(x = "Period", value = "Period"),
+setMethod(
+  "[<-", signature(x = "Period", value = "Period"),
   function(x, i, j, ..., value) {
     x@.Data[i] <- value@.Data
     x@year[i] <- value@year
@@ -247,31 +280,34 @@ setMethod("[<-", signature(x = "Period", value = "Period"),
     x@hour[i] <- value@hour
     x@minute[i] <- value@minute
     x
-})
+  }
+)
 
 #' @export
-setMethod("[[<-", signature(x = "Period", value = "Period"),
-          function(x, i, j, ..., value) {
-            x@.Data[i] <- value@.Data
-            x@year[i] <- value@year
-            x@month[i] <- value@month
-            x@day[i] <- value@day
-            x@hour[i] <- value@hour
-            x@minute[i] <- value@minute
-            x
-})
+setMethod(
+  "[[<-", signature(x = "Period", value = "Period"),
+  function(x, i, j, ..., value) {
+    x@.Data[i] <- value@.Data
+    x@year[i] <- value@year
+    x@month[i] <- value@month
+    x@day[i] <- value@day
+    x@hour[i] <- value@hour
+    x@minute[i] <- value@minute
+    x
+  }
+)
 
 #' @export
 setMethod("$", signature(x = "Period"), function(x, name) {
   if (name == "second") name <- ".Data"
-    slot(x, name)
+  slot(x, name)
 })
 
 #' @export
 setMethod("$<-", signature(x = "Period"), function(x, name, value) {
   if (name == "second") name <- ".Data"
-    slot(x, name) <- rep_len(value, length(x))
-    x
+  slot(x, name) <- rep_len(value, length(x))
+  x
 })
 
 #' Create or parse period objects
@@ -298,14 +334,10 @@ setMethod("$<-", signature(x = "Period"), function(x, name, value) {
 #' to and subtracted to date-times to create a user interface similar to object
 #' oriented programming.
 #'
-#' Note: Arithmetic with periods can results in undefined behavior when
-#' non-existent dates are involved (such as February 29th). Please see
-#' [Period-class] for more details and \code{\link{\%m+\%}} and
-#' [add_with_rollback()] for alternative operations. Note: Arithmetic with
-#' periods can results in undefined behavior when non-existent dates are
-#' involved (such as February 29th in non-leap years). Please see [Period-class]
-#' for more details and \code{\link{\%m+\%}} and [add_with_rollback()] for
-#' alternative operations.
+#' Note: Arithmetic with periods can result in undefined behavior when
+#' non-existent dates are involved (such as February 29th in non-leap years).
+#' Please see [Period-class] for more details and \code{\link{\%m+\%}} and
+#' [add_with_rollback()] for alternative operations.
 #'
 #' @name period
 #' @aliases periods
@@ -343,7 +375,7 @@ setMethod("$<-", signature(x = "Period"), function(x, name, value) {
 #'
 #' ### Units as arguments
 #'
-#' period (second = 90, minute = 5)
+#' period(second = 90, minute = 5)
 #' period(day = -1)
 #' period(second = 3, minute = 1, hour = 2, day = 13, week = 1)
 #' period(hour = 1, minute = -60)
@@ -391,89 +423,93 @@ setMethod("$<-", signature(x = "Period"), function(x, name, value) {
 #' y + months(0:11)
 #'
 #' # compare DST handling to durations
-#' boundary <- ymd_hms("2009-03-08 01:59:59", tz="America/Chicago")
+#' boundary <- ymd_hms("2009-03-08 01:59:59", tz = "America/Chicago")
 #' boundary + days(1) # period
 #' boundary + ddays(1) # duration
 #' @export
 period <- function(num = NULL, units = "second", ...) {
-  nums <- list(...)
   if (is.character(num)) {
     parse_period(num)
-  } else if (!is.null(num) && length(nums) > 0) {
-    c(.period_from_num(num, units), .period_from_units(nums))
-  } else if (!is.null(num)) {
-    .period_from_num(num, units)
-  } else if (length(nums)) {
-    .period_from_units(nums)
   } else {
-    stop("No valid values have been passed to 'period' constructor")
+    out1 <- .period_from_num(num, units)
+    out2 <- .period_from_units(list(...))
+    if (is.null(out1) && is.null(out2)) {
+      new("Period", numeric())
+    } else if (is.null(out1)) {
+      out2
+    } else if (is.null(out2)) {
+      out1
+    } else {
+      c(out1, out2)
+    }
   }
 }
 
 parse_period <- function(x) {
   out <- .Call(C_parse_period, as.character(x))
   new("Period",
-      out[1, ],
-      minute = out[2, ],
-      hour   = out[3, ],
-      day    = out[4, ] + 7L*out[5, ],
-      month  = out[6, ],
-      year   = out[7, ])
+    out[1, ],
+    minute = out[2, ],
+    hour   = out[3, ],
+    day    = out[4, ] + 7L * out[5, ],
+    month  = out[6, ],
+    year   = out[7, ]
+  )
 }
 
 .period_from_num <- function(num, units) {
-
-  if (!is.numeric(num)) {
-    stop(sprintf("First argument to `period` constructor must be character or numeric. Supplied object of class '%s'", class(num)))
+  if (length(num) == 0) {
+    return(NULL)
   }
 
-  if (is.interval(num))
-    stop("Interval objects cannot be used as input to 'period' constructor. Plese use 'as.period'.")
+  if (!is.numeric(num)) {
+    stop(sprintf("First argument to `period()` constructor must be character or numeric. Supplied object of class '%s'", class(num)))
+  }
 
-  if (length(units) %% length(num) != 0)
-    stop("Arguments `num` and `units` must have same length")
+  ## qucik check for common wrongdoings: #462
+  if (inherits(num, c("Interval", "Duration"))) {
+    stop("Interval or Durations objects cannot be used as input to 'period()' constructor. Plese use 'as.period()'.")
+  }
 
-  num <- num + rep(0, length(units))
-  unit <- standardise_date_names(units)
-  pieces <- setNames(as.list(num), unit)
+  stop_incompatible_size(num, units)
+  if (length(units) > length(num)) {
+    num <- rep_len(num, length(units))
+  }
 
-  defaults <- list(second = 0, minute = 0, hour = 0, day = 0, week = 0,
-                   month = 0, year = 0)
-  pieces <- c(pieces, defaults[setdiff(names(defaults), names(pieces))])
-  pieces$day <- pieces$day + 7 * pieces$week
-
-  new("Period", pieces$second, year = pieces$year, month = pieces$month,
-      day = pieces$day, hour = pieces$hour, minute = pieces$minute)
+  .period_from_units(structure(num, names = rep_len(units, length(num))))
 }
 
-.period_from_units <- function(units) {
-  pieces <- data.frame(lapply(units, as.numeric))
+.period_from_units <- function(pieces) {
+  if (length(pieces) == 0) {
+    return(NULL)
+  }
 
-  ## fixme: syncronize this with the initialize method
-  names(pieces) <- standardise_date_names(names(pieces))
-  defaults <- data.frame(
-    second = 0, minute = 0, hour = 0, day = 0, week = 0,
-    month = 0, year = 0
+  if (!is.numeric(pieces)) {
+    pieces <- lapply(pieces, as.numeric)
+  }
+
+  out <- list(
+    second = 0, minute = 0, hour = 0, day = 0,
+    week = 0, month = 0, year = 0
   )
 
-  if (nrow(pieces) == 0) defaults <- defaults[0, ]
+  unit <- standardise_date_names(names(pieces))
+  for (i in seq_along(unit)) {
+    nm <- unit[[i]]
+    out[[nm]] <- out[[nm]] + pieces[[i]]
+  }
+  out$day <- out$day + 7 * out$week
 
-  pieces <- cbind(pieces, defaults[setdiff(names(defaults), names(pieces))])
-  ## pieces <- pieces[c("year", "month", "week", "day", "hour", "minute", "second")]
-
-  pieces$day <- pieces$day + pieces$week * 7
-
-  na <- is.na(rowSums(pieces))
-  pieces$second[na] <- NA ## if any of supplied pieces is NA whole vector should be NA
-
-  new("Period", pieces$second, year = pieces$year, month = pieces$month,
-      day = pieces$day, hour = pieces$hour, minute = pieces$minute)
+  new("Period", out$second,
+    year = out$year, month = out$month,
+    day = out$day, hour = out$hour, minute = out$minute
+  )
 }
 
 #' @rdname period
 #' @examples
 #' is.period(as.Date("2009-08-03")) # FALSE
-#' is.period(period(months= 1, days = 15)) # TRUE
+#' is.period(period(months = 1, days = 15)) # TRUE
 #' @export
 is.period <- function(x) is(x, "Period")
 
@@ -491,13 +527,13 @@ weeks <- function(x = 1) period(week = x)
 #' @rdname period
 years <- function(x = 1) period(year = x)
 #' @rdname period
-milliseconds <- function(x = 1) seconds(x/1000)
+milliseconds <- function(x = 1) seconds(x / 1000)
 #' @rdname period
-microseconds <- function(x = 1) seconds(x/1000000)
+microseconds <- function(x = 1) seconds(x / 1000000)
 #' @rdname period
-nanoseconds <- function(x = 1) seconds(x/1e9)
+nanoseconds <- function(x = 1) seconds(x / 1e9)
 #' @rdname period
-picoseconds <- function(x = 1) seconds(x/1e12)
+picoseconds <- function(x = 1) seconds(x / 1e12)
 
 #' @rdname period
 #' @export
@@ -533,9 +569,6 @@ seconds_to_period <- function(x) {
   remainder <- abs(span)
   newper <- period(second = rep(0, length(x)))
 
-  ## slot(newper, "year") <- remainder %/% (3600 * 24 * 365.25)
-  ## remainder <- remainder %% (3600 * 24 * 365.25)
-
   slot(newper, "day") <- remainder %/% (3600 * 24)
   remainder <- remainder %% (3600 * 24)
 
@@ -558,76 +591,106 @@ summary.Period <- function(object, ...) {
   qq <- c(qq[1L:3L], mean(persecs), qq[4L:5L])
   qq <- seconds_to_period(qq)
   qq <- as.character(qq)
-  names(qq) <- c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.",
-                 "Max.")
-  if (any(nas))
+  names(qq) <- c(
+    "Min.", "1st Qu.", "Median", "Mean", "3rd Qu.",
+    "Max."
+  )
+  if (any(nas)) {
     c(qq, `NA's` = sum(nas))
-  else qq
+  } else {
+    qq
+  }
 }
 
 #' @export
-setMethod("Arith", signature(e1 = "Period", e2 = "ANY"), undefined_arithmetic)
+setMethod("Arith", signature(e1 = "Period", e2 = "ANY"), function(e1, e2) {
+  stop_incompatible_classes(e1, e2, .Generic)
+})
 
 #' @export
-setMethod("Arith", signature(e1 = "ANY", e2 = "Period"), undefined_arithmetic)
+setMethod("Arith", signature(e1 = "ANY", e2 = "Period"), function(e1, e2) {
+  stop_incompatible_classes(e1, e2, .Generic)
+})
 
 ## duration is.numeric. So we need these explicits here:
 #' @export
-setMethod("Arith", signature(e1 = "Duration", e2 = "Period"), undefined_arithmetic)
+setMethod("Arith", signature(e1 = "Duration", e2 = "Period"), function(e1, e2) {
+  stop_incompatible_classes(e1, e2, .Generic)
+})
 
 #' @export
-setMethod("Arith", signature(e1 = "Period", e2 = "Duration"), undefined_arithmetic)
+setMethod("Arith", signature(e1 = "Period", e2 = "Duration"), function(e1, e2) {
+  stop_incompatible_classes(e1, e2, .Generic)
+})
 
 #' @export
-setMethod("Compare", signature(e1 = "Period", e2 = "Period"),
-          function(e1, e2) {
-            callGeneric(period_to_seconds(e1), period_to_seconds(e2))
-          })
+setMethod(
+  "Compare", signature(e1 = "Period", e2 = "Period"),
+  function(e1, e2) {
+    callGeneric(period_to_seconds(e1), period_to_seconds(e2))
+  }
+)
 
 #' @export
-setMethod("Compare", signature(e1 = "Period", e2 = "character"),
-          function(e1, e2) {
-            callGeneric(e1, as.period(e2))
-          })
+setMethod(
+  "Compare", signature(e1 = "Period", e2 = "character"),
+  function(e1, e2) {
+    callGeneric(e1, as.period(e2))
+  }
+)
 
 #' @export
-setMethod("Compare", signature(e1 = "character", e2 = "Period"),
-          function(e1, e2) {
-            callGeneric(as.period(e1), e2)
-          })
+setMethod(
+  "Compare", signature(e1 = "character", e2 = "Period"),
+  function(e1, e2) {
+    callGeneric(as.period(e1), e2)
+  }
+)
 
 #' @export
-setMethod("Compare", signature(e1 = "Period", e2 = "Duration"),
-          function(e1, e2) {
-            callGeneric(as.duration(e1), e2)
-          })
+setMethod(
+  "Compare", signature(e1 = "Period", e2 = "Duration"),
+  function(e1, e2) {
+    callGeneric(as.duration(e1), e2)
+  }
+)
 
 #' @export
-setMethod("Compare", signature(e1 = "Duration", e2 = "Period"),
-          function(e1, e2) {
-            callGeneric(e1, as.duration(e2))
-          })
+setMethod(
+  "Compare", signature(e1 = "Duration", e2 = "Period"),
+  function(e1, e2) {
+    callGeneric(e1, as.duration(e2))
+  }
+)
 
 #' @export
-setMethod("Compare", signature(e1 = "Period", e2 = "numeric"),
-          function(e1, e2) {
-            callGeneric(as.numeric(e1, "secs"), e2)
-          })
+setMethod(
+  "Compare", signature(e1 = "Period", e2 = "numeric"),
+  function(e1, e2) {
+    callGeneric(as.numeric(e1, "secs"), e2)
+  }
+)
 
 #' @export
-setMethod("Compare", signature(e1 = "numeric", e2 = "Period"),
-          function(e1, e2) {
-            callGeneric(e1, as.numeric(e2, "secs"))
-          })
+setMethod(
+  "Compare", signature(e1 = "numeric", e2 = "Period"),
+  function(e1, e2) {
+    callGeneric(e1, as.numeric(e2, "secs"))
+  }
+)
 
 #' @export
-setMethod("Compare", c(e1 = "Period", e2 = "difftime"),
-          function(e1, e2) {
-            callGeneric(as.numeric(e1, units = "secs"), as.numeric(e2, units = "secs"))
-          })
+setMethod(
+  "Compare", c(e1 = "Period", e2 = "difftime"),
+  function(e1, e2) {
+    callGeneric(as.numeric(e1, units = "secs"), as.numeric(e2, units = "secs"))
+  }
+)
 
 #' @export
-setMethod("Compare", c(e1 = "difftime", e2 = "Period"),
-          function(e1, e2) {
-            callGeneric(as.numeric(e1, units = "secs"), as.numeric(e2, units = "secs"))
-          })
+setMethod(
+  "Compare", c(e1 = "difftime", e2 = "Period"),
+  function(e1, e2) {
+    callGeneric(as.numeric(e1, units = "secs"), as.numeric(e2, units = "secs"))
+  }
+)
